#pragma rtGlobals=1		// Use modern global access method.

Function RatioImg0Style()
	ModifyGraph/Z rgb[0]=(0,0,0),rgb[1]=(65535,0,52428),rgb[2]=(32768,32770,65535)	// Tweaks to improve display/printing
	ModifyGraph/Z grid(left)=2																	// Tweaks to improve display/printing
	ModifyGraph/Z nticks(bottom)=10,nticks(right)=10,nticks(left)=10							// Tweaks to improve display/printing
	ModifyGraph/Z highTrip(right)=100000														// Tweaks to improve display/printing
	ModifyGraph/Z lSize=0.5																		// Tweaks to improve display/printing
	ModifyGraph/Z btLen=5																			// Tweaks to improve display/printing
	ModifyGraph/Z btThick=0.5																		// Tweaks to improve display/printing
	ModifyGraph/Z stLen=5																			// Tweaks to improve display/printing
	ModifyGraph/Z stThick=0.5																		// Tweaks to improve display/printing
	ModifyGraph/Z ttLen=5																			// Tweaks to improve display/printing
	ModifyGraph/Z ttThick=0.5																		// Tweaks to improve display/printing
	ModifyGraph/Z ftLen=5																			// Tweaks to improve display/printing
	ModifyGraph/Z ftThick=0.5																		// Tweaks to improve display/printing
	SetAxis/Z left .4,4				// Left axis scaling is hard-wired for typical SBFI (sodium-indicator dye) ratios. For Fura-2 you should probably increase the upper limit to 8 or 10.
	SetAxis/Z/A/E=1 right			// A and B data are autoscaled from zero on the right axis.
	ModifyGraph grid(left)=1
	ModifyGraph fSize=9
End

//	RatioImgGraphWindowHook(infoStr)
//	This routine is the hook function for RatioImg graphs. It is called when certain events
//	(e.g., activate, deactivate, kill) occur.
Function RatioImgGraphWindowHook(infoStr)
	String infoStr
	
	Variable statusCode = 0									// 0 if nothing done, else 1
	
	String eventKey = StringByKey("EVENT", infoStr)		// Determine which event happened.
	
	if (CmpStr(eventKey, "activate") == 0)
		// If the Set Period Info control panel is displayed, we need to update what it is displaying.
		UpdateSetPeriodInfoPanel()
 	endif

	return statusCode
End

//	SetRatioImgGraphTitleAndLabel(dataFolder, cell)
//	Sets the window title and the label which identify the cell being displayed in the active graph
//	The active graph is assumed to be a RatioImg graph.
Function SetRatioImgGraphTitleAndLabel(dataFolder, cell)
	String dataFolder										// The path to data folder containing the RatioImg data being displayed.
	Variable cell											// The number of the cell being displayed.

	String dataFolderName, graphTitle
	
	String graphName = WinName(0, 1)					// The name of the graph to operate on.
	dataFolderName = GetLeafName(dataFolder)				// Get just the run name.
	sprintf graphTitle, "%s.%02d", dataFolderName, cell
	DoWindow/T $graphName, graphTitle
	Textbox/C/N=cellLabel/F=0/M/A=LT graphTitle
End

Function CreateRatioImgGraph(dataFolder, cell)
	String dataFolder										// The RatioImg data folder.
	Variable cell											// The cell to be graphed.

	String originalCurrentDataFolder = GetDataFolder(1)
	SetDataFolder(dataFolder)

	if (cell < 1)  // Clip cell.
		cell = 2
	endif

	NVAR gNumDataPairs=gNumDataPairs					//Create a reference to the global variable in the current data folder.
	if (cell > gNumDataPairs)								// Clip cell.
		cell = gNumDataPairs
	endif

	String aName, bName, rName, graphName
	Variable aExists, bExists, rExists
															// This is just a convenient structure to break from in the event of an error. It is not being used to loop here.
	do														
		aName = "A" + num2istr(cell)						// e.g., "A2"
		Wave wa = $aName
			// Get a reference to the A wave
		aExists = WaveExists(wa)

		bName = "B" + num2istr(cell)						// e.g., "B2"
		Wave wb = $bName
			// Get a reference to the B wave
		bExists = WaveExists(wb)

		rName = "R" + num2istr(cell)						// Ratio wave will be named R2
		Wave wr = $rName
			// Get a reference to the R wave
		rExists = WaveExists(wr)

		if (!rExists)
			break											// Should never happen.
		endif

		Display/W=(5,42,504,335) wr
		if (aExists)
			AppendToGraph/R wa
		endif
		if (bExists)
			AppendToGraph/R wb
		endif

		graphName = UniqueName("RatioImgGraph", 6, 0 )	//RatioImgGraph0, RatioImgGraph1 . . .
		DoWindow/C $graphName
		SetRatioImgGraphTitleAndLabel(dataFolder, cell)
		RatioImg0Style()
		SetWindow $graphName hook=RatioImgGraphWindowHook
		ShowInfo
	while(0)												// Quit after the first iteration.
	
	SetDataFolder(originalCurrentDataFolder)				//Restore current data folder.
End

Function CreateRatioImgAvgGraph()

	String graphName
	Wave wr = R_Average
	Display/W=(305,342,804,635) wr
	graphName = UniqueName("RatioImgGraph", 6, 0 )	//RatioImgGraph0, RatioImgGraph1 . . .
	DoWindow/C $graphName
	RatioImg0Style()
	SetAxis/A
	ShowInfo
	
End

//	CreateRatioImgGraphs(dataFolder, startCell, endCell)
//	dataFolder is a RatioImg data folder containing A/B/R data loaded to be graphed.
//	This routine creates a graph of A/B/R data for each cell from startCell to endCell.
Function CreateRatioImgGraphs(dataFolder, startCell, endCell)
	String dataFolder									// The RatioImg data folder.
	Variable startCell, endCell							// The cells to be graphed.
	
	String originalCurrentDataFolder = GetDataFolder(1)
	SetDataFolder(dataFolder)
	
	if (startCell < 1)									// Clip startCell.
		startCell = 2									
	endif
	
	NVAR gNumDataPairs=gNumDataPairs				// Create a reference to the global variable in the current data folder.
	if (endCell > gNumDataPairs)						// Clip endCell.
		endCell = gNumDataPairs
	endif

	String aName, bName, rName
	Variable cell = startCell
	do
		CreateRatioImgGraph(dataFolder, cell)
		cell += 1
	while(cell <= endCell)

	SetDataFolder(originalCurrentDataFolder)			// Restore current data folder.
End

Function MakeAvgWaveGraph(ctrlName)
	String ctrlName
	
	NVAR ratiosCorrected=gRatiosCorrected				// Access the global variable named gRatiosCorrected.
	if (ratiosCorrected != 1)
		String temp
		sprintf temp, "Please correct the values and select a Run data folder in the Data Browser, then try again."
		DoAlert 0, temp
		return -1
	endif
	
	CreateRatioImgAvgGraph() 							// Create a window displaying the averaged wave.
		
End


//	CreateRatioImgGraphsFromList(dataFolderList)
//	dataFolderList contains a semicolon-separated list of data folders containing A/B/R data loaded to be
//	graphed. However, if dataFolderList is "", this routine uses the RatioImg data folders selected in the Data Browser.
//	This routine creates a graph displaying ROI number 2 for each set of A/B/R data.
//	The user can then use control panels to browse other cells.
Function CreateRatioImgGraphsFromList(dataFolderList)
	String dataFolderList
	
	if (strlen(dataFolderList) == 0)
		dataFolderList = GetSelectedRunDataFolders(0)
	endif

	if (strlen(dataFolderList) == 0)
		DoAlert 0, "Select one or more run data folders in the Data Browser before invoking this procedure"
		return -1
	endif
	
	// Next, call CreateRatioImgGraphs
		
	String dataFolder
	Variable i = 0
	do
		dataFolder = StringFromList(i, dataFolderList, ";")
		if (strlen(dataFolder) == 0)
			break												// No more run data folders.
		endif
		CreateRatioImgGraphs(dataFolder, 2, 2)					
		i += 1
	while(1)
	
	 //  Create windows for calibration & browsing, make the RatioImg Notebook

 	DisplaySetPeriodInfoPanel()
	MakeNotebook("","")
	DisplayRatioImgRunBrowserPanel()
		
	return 0	
End

Function CantFindRunAlert()
	String message
	String graphName = WinName(0, 1)
	if (strlen(graphName) == 0)
		message = "There are no graphs"
	else
		message = "Can't find run data in the active graph"
	endif
	DoAlert 0, message
End

//	NextOrPrevCellButtonProc(next)
//	Displays the next or previous cell in the run data folder in the active graph which is assumed to be a RatioImg graph.
//	If next is zero, it displays the previous cell, if non-zero, it displays the next cell.
Function NextOrPrevCellButtonProc(next)
	Variable next
	
	// Get the data folder for the current run.
	String dataFolder = GetDataFolderForDisplayedRun(1)		// Includes a trailing colon.
	if (strlen(dataFolder) == 0)
		CantFindRunAlert()
		return -1
	endif
	
	Variable i
	
	// Get a list of the waves to be replaced.
	String oldWaveList = ""
	i = 0
	do
		WAVE w = WaveRefIndexed("", i, 1)						// Get a reference to the next wave in the top graph.
		if (!WaveExists(w))
			break												// No more waves.
		endif
		if (IsRatioImgWave(w))
			oldWaveList += GetWavesDataFolder(w, 2) + ";"	// This includes the name of the wave, not just the data folder name.
		endif
		i += 1
	while(1)
	
	if (strlen(oldWaveList) == 0)
		return -1												// No waves were found to replace.
	endif
	
	// Replace each wave with the next or previous wave of the same type (e.g., A2->A3, B2->B3, R2->R3).
	String traceName, oldWave
	i = 0
	do
		oldWave = StringFromList(i, oldWaveList, ";")
		if (strlen(oldWave) == 0)							// No more waves to replace.
			break
		endif
		Wave oldW = $oldWave								// Get a reference to the old wave.
		traceName = WaveToTraceName("", oldW)			// Get the trace name for the wave.
		if (strlen(traceName) == 0)
			break											// Should never happen.
		endif
		if (next)
			Wave newW = $GetNextRatioImgWave(oldW, 1)
		else
			Wave newW = $GetPrevRatioImgWave(oldW, 1)
		endif
		if (WaveExists(newW))							// It should always exist.
			ReplaceWave trace=$traceName, newW
		endif
		i += 1
	while(1)
	
	Variable cell = GetRatioImgWaveCellNumber(newW)
	String graphName = WinName(0, 1)					// Get name of top graph.
	SetRatioImgGraphTitleAndLabel(dataFolder, cell)
End

//	NextCellButtonProc(ctrlName)
//	This is the action procedure for the Next Cell button in the RatioImg Data Browser control panel.
Function NextCellButtonProc(ctrlName) : ButtonControl
	String ctrlName
	
	NextOrPrevCellButtonProc(1)
End

//	PrevCellButtonProc(ctrlName)
//	This is the action procedure for the Prev Cell button in the RatioImg Data Browser control panel.
Function PrevCellButtonProc(ctrlName) : ButtonControl
	String ctrlName
	
	NextOrPrevCellButtonProc(0)
End


//	NextOrPrevRunButtonProc(next)
//	Displays the next or previous run in the active graph which is assumed to be a RatioImg graph.
//	If next is zero, it displays the previous run, if non-zero, it displays the next run.
Function NextOrPrevRunButtonProc(next)
	Variable next
	
	// Get the data folder for the current run.
	String dataFolder = GetDataFolderForDisplayedRun(1)		// Includes a trailing colon.
	if (strlen(dataFolder) == 0)
		CantFindRunAlert()
		return -1
	endif
	
	Variable i
	
	// Get a list of the waves to be replaced.
	String oldWaveList = ""
	i = 0
	do
		WAVE w = WaveRefIndexed("", i, 1)						// Get a reference to the next wave in the top graph.
		if (!WaveExists(w))
			break												// No more waves.
		endif
		if (IsRatioImgWave(w))
			oldWaveList += GetWavesDataFolder(w, 2) + ";"	// This includes the name of the wave, not just the data folder name.
		endif
		i += 1
	while(1)
	
	if (strlen(oldWaveList) == 0)
		return -1												// No waves were found to replace.
	endif
	
	// Replace each wave with a corresponding wave from the next or previous run.
	String newDataFolder
	String traceName, oldWave, newName
	i = 0
	do
		oldWave = StringFromList(i, oldWaveList, ";")
		if (strlen(oldWave) == 0)							// No more waves to replace.
			break
		endif
		Wave oldW = $oldWave								// Get a reference to the old wave.
		traceName = WaveToTraceName("", oldW)			// Get the trace name for the wave.
		if (strlen(traceName) == 0)
			break											// Should never happen.
		endif
		if (next)
			newDataFolder = GetNextRunDataFolder(dataFolder, 1)	// Does include trailing colon.
		else
			newDataFolder = GetPrevRunDataFolder(dataFolder, 1)	// Does include trailing colon.
		endif
		newName = GetLeafName(oldWave)
		Wave newW = $(newDataFolder + ":" + newName)	// Try to display the same cell from the new run.
		if (!WaveExists(newW))							// Does this run have a corresponding cell?
			newName = newName[0] + "2"					// Go back to cell 2.
			Wave newW = $(newDataFolder + ":" + newName)
		endif
		if (WaveExists(newW))							// It should always exist.
			ReplaceWave trace=$traceName, newW
		endif
		i += 1
	while(1)
	
	Variable cell = GetRatioImgWaveCellNumber(newW)
	String graphName = WinName(0, 1)					// Get name of top graph.
	SetRatioImgGraphTitleAndLabel(newDataFolder, cell)
End

//	NextRunButtonProc(ctrlName)
//	This is the action procedure for the Next Run button in the RatioImg Data Browser control panel.
Function NextRunButtonProc(ctrlName) : ButtonControl
	String ctrlName

	NextOrPrevRunButtonProc(1)
End

//	PrevRunButtonProc(ctrlName)
//	This is the action procedure for the Prev Run button in the RatioImg Data Browser control panel.
Function PrevRunButtonProc(ctrlName) : ButtonControl
	String ctrlName

	NextOrPrevRunButtonProc(0)
End

Function RemoveLineFits(ctrlName)	: ButtonControl	// Removes and kills waves named fit_<whatever>
	String ctrlName
	String graphName = WinName(0, 1)				// Name of the top graph.
	if (!IsRatioImgGraph(graphName))
		DoAlert 0, "Activate a RatioImg graph first."
		return -1
	endif

	Variable i
	String name, traceName
	i = 0
	do
		Wave w = WaveRefIndexed("", i, 1)
		if (!WaveExists(w))
			break							// All done.
		endif
		name = NameOfWave(w)
		if (CmpStr(name[0,3], "fit_") == 0)
			traceName = WaveToTraceName("", w)
			if (strlen(traceName) > 0)					// Should always be > 0.
				RemoveFromGraph $traceName
				KillWaves/Z w
				i -= 1
			endif
		endif
		i += 1
	while(1)
End

//	Adds the specified graph to the specified notebook.
//	If nb is "", it adds it to a notebook named RatioImgNotebook, creating the
//	notebook if it does not yet exist.
//	If graphName is "", it uses the top graph.
//	Usually, this would be invoked with the command:
//		AddGraphToNotebook("", "")
Function AddGraphToNotebook(nb, graphName)
	String nb
	String graphName
	
	String message
	
	if (strlen(graphName) == 0)
		graphName = WinName(0, 1)		// Get name of top graph.
	endif
	if (strlen(graphName) == 0)
		DoAlert 0, "There is no graph to add to the notebook"
		return -1
	endif
	
	DoWindow $graphName
	if (V_flag == 0)
		sprintf message, "There is no graph named %s", graphName
		DoAlert 0, message
		return -1
	endif
	
	if (strlen(nb) == 0)
		nb = "RatioImgNotebook"
		DoWindow $nb
		if (V_flag == 0)
			// The notebook is created invisible. Use the Windows menu to show it.
			NewNotebook/F=1/N=$nb/V=0 as "RatioImg Notebook" 
			Notebook $nb showRuler=0
		endif
	endif
	
	DoWindow $nb
	if (V_flag == 0)
		sprintf message, "There is no notebook named %s", nb
		DoAlert 0, message
		return -1
	endif
	
	// Add the graph to the notebook.	
	
	Notebook $nb picture={$graphName(0, 0, 400, 200), -2, 1}
	ModifyGraph width=276,height=150,margin(top)=12,margin(left)=60,margin(bottom)=36,margin(right)=60 	//Resizes graph for copying
	Notebook $nb findText={"", 1}	// Scroll new picture to view.
	ModifyGraph margin=0,width=0,height=0		// "Unlocks" the graph size (secondary effect of re-sizing it).
End

Function callAddGraphToNotebook(ctrlName) : ButtonControl
	string ctrlName
	AddGraphToNotebook("","")
End

Function MakeNotebook(nb,graphName)
	String nb
	String graphName
	
	String message
	
	if (strlen(graphName) == 0)
		graphName = WinName(0, 1)		// Get name of top graph.
	endif
	if (strlen(graphName) == 0)
		DoAlert 0, "There is no graph to add to the notebook"
		return -1
	endif
	
	DoWindow $graphName
	if (V_flag == 0)
		sprintf message, "There is no graph named %s", graphName
		DoAlert 0, message
		return -1
	endif
	
	if (strlen(nb) == 0)
		nb = "RatioImgNotebook"
		DoWindow $nb
		if (V_flag == 0)
			// The notebook is created invisible. Use the Windows menu to show it.
			NewNotebook/N=$nb/F=1/V=0 as "RatioImg Notebook" 						// The notebook is created as invisible. Use the Windows menu to show it.
			Notebook $nb defaultTab=36, statusWidth=238, pageMargins={129,36,14,36}	
			Notebook $nb newRuler=Normal, justification=0, margins={0,0,468}, spacing={0,0,0}, tabs={}, rulerDefaults={"Geneva",10,0,(0,0,0)}
			Notebook $nb ruler=Normal; Notebook $nb  margins={0,0,814}
		endif
	endif
	
	DoWindow $nb
	if (V_flag == 0)
		sprintf message, "There is no notebook named %s", nb
		DoAlert 0, message
		return -1
	endif	
End

Proc GetNumberOfMeasuresToDelete(numberOfMeasures)
	Variable numberOfMeasures = NumVarOrDefault("gLim",0)
	Prompt numberOfMeasures, "Enter the number of measurements to delete from the last made (n - 1):"
	
	Variable/G gLim = numberOfMeasures
End

Function MultipleDelLast(ctrlName) : ButtonControl
	String ctrlName

	// Display dialog to get number of graphs.
	
	String dfSav = GetDataFolder(1)						// Save current folder location
	SetDataFolder root:Packages:RatioImg				// Set folder for gLim
	
	Execute "GetNumberOfMeasuresToDelete()"
	NVAR lim = gLim		// gLim is created by GetNumberOfMeasuresToDelete.
	
	Silent 1
	Variable i = 1
	Do
		if (i > lim)
			break
		endif
		DeleteLastMeasurement("")
		i += 1
	while (1)
	Silent 0
	printf "Deleted last %d measurements.\r", lim
	
	SetDataFolder dfSav									// Restore folder location to original.
	
End

Proc GetNumberOfGraphsToMeasure(numberOfGraphs)
	Variable numberOfGraphs = NumVarOrDefault("gLim",0)
	Prompt numberOfGraphs, "Enter the number of graphs to measure (n - 1):"
	
	Variable/G gLim = numberOfGraphs
End

Function MultipleMeasurements(ctrlName) : ButtonControl
	String ctrlName
	
	String graphName = WinName(0, 1)				// Check that we have a graph available.
	if (!IsRatioImgGraph(graphName))
		DoAlert 0, "Activate a RatioImg graph first."
		return -1
	endif
	
	Wave wA = CsrWaveRef(A)							// Check that cursor A is on the graph.
	if (!WaveExists(wA))
		DoAlert 0, "Put cursor A on a wave first"
		return -1
	endif
	Wave wB = CsrWaveRef(B)							// Check that cursor B is on the graph.
	if (!WaveExists(wB))
		DoAlert 0, "Put cursor B on a wave first"
		return -1
	endif

	// Display dialog to get number of graphs.
	
	String dfSav = GetDataFolder(1)						// Save current folder location
	SetDataFolder root:Packages:RatioImg				// Set folder for gLim
	
	Execute "GetNumberOfGraphsToMeasure()"
	NVAR lim = gLim									// gLim is created by GetNumberOfGraphsToMeasure.
	
	PauseUpdate
	Variable i = 1
	Do
		if (i > lim)
			break
		endif
		AddMeasurementFromCursors("")
		NextOrPrevCellButtonProc(1)
		i += 1
	while (1)
	ResumeUpdate
	ViewMeasurements()
	DoAlert 0, "All done measuring the graphs."
	
	SetDataFolder dfSav									// Restore folder location to original.

End

Proc GetNumberOfGraphsToCopy(numberOfGraphs)
	Variable numberOfGraphs = NumVarOrDefault("gLim",0)
	Prompt numberOfGraphs, "Enter the number of graphs to copy to the notebook (n - 1):"
	
	Variable/G gLim = numberOfGraphs
End

Function MultipleCopyGraphs(ctrlName) : ButtonControl
	String ctrlName

	// Display dialog to get number of graphs.
	
	String dfSav = GetDataFolder(1)						// Save current folder location
	SetDataFolder root:Packages:RatioImg				// Set folder for gLim
	
	Execute "GetNumberOfGraphsToCopy()"
	NVAR lim = gLim									// gLim is created by GetNumberOfGraphsToMeasure.
	
	Variable i = 1
	Do
		if (i > lim)
			break
		endif
		AddGraphToNotebook("", "")
		NextOrPrevCellButtonProc(1)
		DoUpdate					//Forces graph to update, otherwise autoscaling fails and all graphs have the scaling of the first graph.
		i += 1
	while (1)
	DoAlert 0, "All done copying the graphs to the notebook."
	
	SetDataFolder dfSav									// Restore folder location to original.
	
End

Window RatioImgRunBrowser() : Panel
	PauseUpdate; Silent 1		// building window...
	NewPanel /K=1 /W=(458,539,755,628) as "Experiment/ROI Browser"
	SetDrawLayer UserBack
	SetDrawEnv linefgc= (48059,48059,48059),fillfgc= (57906,61602,58830)
	DrawRect 0,-4,305,138
	SetDrawEnv linefgc= (0,0,65535)
	DrawRect 111,-44,183,-22
	SetDrawEnv linefgc= (65535,65535,0)
	DrawRect -123,7,-51,29
	SetDrawEnv linefgc= (61166,61166,61166),fillfgc= (47323,57015,51361)
	DrawRect 105,57,10,79
	SetDrawEnv linefgc= (61166,61166,61166),fillfgc= (47323,57015,51361)
	DrawRect 105,32,10,54
	SetDrawEnv linefgc= (61166,61166,61166),fillfgc= (47323,57015,51361)
	DrawRect 104,7,9,29
	SetDrawEnv linefgc= (65535,65535,65535),fillfgc= (57906,61602,58830)
	DrawRect 110,6,185,56
	SetDrawEnv linefgc= (65535,65535,65535),fillfgc= (57906,61602,58830)
	DrawRect 187,6,262,56
	Button MeasureStore,pos={34,8},size={80,20},proc=AddMeasurementFromCursors,title="Measure"
	Button MeasureStore,help={"Measures Avg, StDev, Slope, etc. between the two cursors and enters them in the Measurements table."}
	Button RemoveLines,pos={206,58},size={80,20},proc=RemoveLineFits,title="Kill Fits"
	Button RemoveLines,help={"Deletes all line-fits from the graph."}
	Button DelLastMeasurement,pos={34,58},size={80,20},proc=DeleteLastMeasurement,title="Del Last"
	Button DelLastMeasurement,help={"Deletes the last set of A-to-B cursor measurements."}
	Button NextCellButton,pos={120,8},size={80,20},proc=NextCellButtonProc,title="Next Cell"
	Button NextCellButton,help={"Displays the A, B and R curves for the next cell in the currently displayed run."}
	Button PrevCellButton,pos={120,33},size={80,20},proc=PrevCellButtonProc,title="Prev Cell"
	Button PrevCellButton,help={"Displays the A, B and R curves for the previous cell in the currently displayed run."}
	Button DelAllMeasuresButton,pos={120,58},size={80,20},proc=DeleteAllMeasurements,title="Clear All"
	Button DelAllMeasuresButton,help={"Deletes all measurements from the Measurements table."}
	Button NextRunButton,pos={206,8},size={80,20},proc=NextRunButtonProc,title="Next Run"
	Button NextRunButton,help={"Displays the A, B and R curves for the next run in the Data Browser."}
	Button PrevRunButton,pos={206,33},size={80,20},proc=PrevRunButtonProc,title="Prev Run"
	Button PrevRunButton,help={"Displays the A, B and R curves for the previous run in the Data Browser."}
	Button AddGraphToNB,pos={34,33},size={80,20},proc=callAddGraphToNotebook,title="Graph to NB"
	Button AddGraphToNB,help={"Copies the currently-displayed graph to the RatioImg Notebook."}
	Button TimesMeasure,pos={11,8},size={20,20},proc=MultipleMeasurements,title="*"
	Button TimesMeasure,help={"Prompts for a number, then makes this number of measurements beginning with the current graph."}
	Button TimesGraphToNB,pos={11,33},size={20,20},proc=MultipleCopyGraphs,title="*"
	Button TimesGraphToNB,help={"Prompts for a number, then copies this number of graphs to the notebook beginning with the current graph."}
	Button TimesDelLast,pos={11,58},size={20,20},proc=MultipleDelLast,title="*"
	Button TimesDelLast,help={"Prompts for a number, then deletes this number of measurements from the Measurements table beginning with the most recent."}
EndMacro

//	DisplayRatioImgRunBrowserPanel()
//	Brings the Set Period Info control panel to the front after creating it if necessary.
Function DisplayRatioImgRunBrowserPanel()
	DoWindow/F RatioImgRunBrowser
	if (V_flag == 0)
		Execute "RatioImgRunBrowser()"		// We need to use Execute because you can't directly call a macro from a function.	
	endif
End

//	AutoSetRatioImgAxisRanges(whichAxes, criteria,  applyTo)
//	HR, 01/13/99 - Added AutoSetRatioImgAxisRanges. Not yet finished or documented.
//	Need to do right axis as well.
Function AutoSetRatioImgAxisRanges(whichAxes, criteria,  applyTo)
	Variable whichAxes						// 1 = left, 2 = right, 3 = both left and right.
	Variable criteria						// 1 = based on currently displayed cell, 2 = based on all cells in currently displayed run, 3 = based on all runs currently selected in the Data Browser.
	Variable applyTo						// 1 = the active graph, 2 = All open RatioImg graphs

	String graphName = WinName(0, 1)
	if (applyTo == 1)
		if (!IsRatioImgGraph(graphName))
			DoAlert 0, "Activate a RatioImg graph before running this procedure"
			return -1
		endif
	endif

	Variable i, j
	Variable doLeft = (whichAxes %& 1)							// Do we want to do the left axis?
	Variable doRight = (whichAxes %& 2)							// Do we want to do the right axis?
	Variable ratioMin, ratioMax
	Variable dataMin, dataMax
	String dataFolder, dataFolderList

	String originalCurrentDataFolder = GetDataFolder(1)
	
	// Find the first ratio wave in the graph.
	if (doLeft)
		i = 0
		do
			Wave w = WaveRefIndexed("", i, 1)
			if (!WaveExists(w))										// No more waves.
				DoAlert 0, "Can not find a ratio wave in the graph"
				return -1
			endif
			if (IsRatioImgWaveOfType(w, "R"))
				Wave ratioWave = w
				break
			endif
			i += 1
		while(1)
	endif
	
	if (doRight)
		// Find the first A and B waves in the graph.
		i = 0
		Variable foundA = 0, foundB = 0
		do
			Wave w = WaveRefIndexed("", i, 1)
			if (!WaveExists(w))										// No more waves.
				DoAlert 0, "Can not find A and waves in the graph"
				return -1
			endif
			if (IsRatioImgWaveOfType(w, "A"))
				Wave AWave = w
				foundA = 1
			endif
			if (IsRatioImgWaveOfType(w, "B"))
				Wave BWave = w
				foundB = 1
			endif
			if (foundA %& foundB)
				break
			endif
			i += 1
		while(1)
	endif
	
	if (criteria == 1)							// Scale axis based on currently displayed cell.
		if (doLeft)
			WaveStats/Q ratioWave
			ratioMin = V_min; ratioMax = V_max
		endif

		if (doRight)
			WaveStats/Q AWave
			dataMin = V_min; dataMax = V_max

			WaveStats/Q BWave
			if (V_min < dataMin)
				dataMin = V_min
			endif
			if (V_max > dataMax)
				dataMax = V_max
			endif
		endif
	endif
	
	if (criteria == 2)							// Scale axis based on all cells in currently displayed run.
		if (doLeft)
			dataFolderList = GetWavesDataFolder(ratioWave, 1)					// Includes trailing colon.
		else
			dataFolderList = GetWavesDataFolder(AWave, 1)						// Includes trailing colon.
		endif
		RemoveTrailingColon(dataFolderList)
	endif
		
	if (criteria == 3)							// Scale axis 3 = based on all runs currently selected in the Data Browser.
		dataFolderList = GetSelectedRunDataFolders(0)
		if (strlen(dataFolderList) == 0)
			DoAlert 0, "No run data folders were selected in the Data Browser"
		endif
	endif

	if ( (criteria == 2) %| (criteria == 3) )
		ratioMin = INF; ratioMax = -INF
		i = 0
		do
			dataFolder = StringFromList(i, dataFolderList, ";")
			if (strlen(dataFolder) == 0)
				break													// No more data folders.
			endif
			SetDataFolder(dataFolder)
			j = 0
			do
				Wave w = WaveRefIndexed("", j, 4)						// Using "" and 4 makes WaveRefIndexed work on all waves in the current data folder.
				if (!WaveExists(w))									// No more waves.
					break
				endif
				
				if (doLeft)
					if (IsRatioImgWaveOfType(w, "R"))					// Is this a ratio wave?
						WaveStats/Q w
						if (V_min < ratioMin)
							ratioMin = V_min
						endif
						if (V_max > ratioMax)
							ratioMax = V_max
						endif
					endif
				endif
				
				if (doRight)
					if ( IsRatioImgWaveOfType(w, "A") %| IsRatioImgWaveOfType(w, "B") )		//  Is this a data wave?
						WaveStats/Q w
						if (V_min < dataMin)
							dataMin = V_min
						endif
						if (V_max > dataMax)
							dataMax = V_max
						endif
					endif
				endif
				
				j += 1
			while(1)			// End of the inner loop.
			i += 1
		while(1)				// End of the outer loop.
	endif
	
	if (applyTo == 1)
		// Apply to the active graph.
		if (doLeft)
			SetAxis left ratioMin, ratioMax
		endif
		if (doRight)
			SetAxis right dataMin, dataMax
		endif
	else
		// Apply to all open RatioImg graphs.
		
		// Make a list of RatioImg graphs.
		String graphList = ""
		i = 0
		do
			graphName = WinName(i, 1)
			if (strlen(graphName) == 0)
				break
			endif
			if (IsRatioImgGraph(graphName))
				graphList += graphName + ";"
			endif
			i += 1
		while(1)
		
		// Now do the SetAxis for each graph. Unfortunately, this means we have to activate each graph.
		// We do this in reverse order so that order of the RatioImg graphs will not be changed by this procedure.
		Variable numGraphs = 0
		do
			graphName = StringFromList(numGraphs, graphList, ";")
			if (strlen(graphName) == 0)
				break
			endif
			numGraphs += 1
		while(1)

		if (numGraphs > 0)
			i = numGraphs - 1
			do
				graphName = StringFromList(i, graphList, ";")
				DoWindow/F $graphName
				if (doLeft)
					SetAxis left ratioMin, ratioMax
				endif
				if (doRight)
					SetAxis right dataMin, dataMax
				endif
				i -= 1
			while(i >= 0)
		endif
	endif
	
	SetDataFolder(originalCurrentDataFolder)			// Restore current data folder.
End

//	MAutoSetRatioImgAxisRanges(whichAxes, whatCriteria, extraMargin, applyTo)
//	Displays a dialog in which the user can specify how to scale the left and right axes.
//	This is implemented as a proc (a form of macro) because functions do not have the
//	ability to display the missing parameter dialog.
Proc MAutoSetRatioImgAxisRanges(whichAxes, criteria,  applyTo)
	Variable whichAxes = 3
	Prompt whichAxes, "Axes to be set", popup "Left;Right;Left and Right"
	Variable criteria = 2
	Prompt criteria, "Based on", popup "The currently displayed cell;All cells in the currently displayed run;All runs selected in the Data Browser"
	Variable applyTo = 1
	Prompt applyTo, "Apply to", popup "The active graph;All RatioImg graphs"
	
	AutoSetRatioImgAxisRanges(whichAxes, criteria,  applyTo)
End
