#pragma rtGlobals=1		// Use modern global access method.

#include "RatioImg Main"

Function ViewMeasurements()				// Displays a table of measurements.
	String originalCurrentDataFolder
	originalCurrentDataFolder = GetDataFolder(1)
	SetDataFolder root:Packages:RatioImg:Measurements

	DoWindow/F MeasurementsTable
	if (V_flag == 0)						// Table does not exist?
		Edit Name, Start, Stop, Average, StdDev, Slope, Pearson, YStart, YStop
		DoWindow/C MeasurementsTable						// Set the table's name.
		DoWindow/T MeasurementsTable, "Measurements"		// Set the table's title.
	endif

	SetDataFolder originalCurrentDataFolder
End

Function SortMeasurements()
	String originalCurrentDataFolder
	originalCurrentDataFolder = GetDataFolder(1)
	SetDataFolder root:Packages:RatioImg:Measurements
	
	// We need to modify the names to that "R2" sorts before "R13".
	// We do this by creating a temp wave in which we change
	// ".<letter><digit>" to ".<letter>0<digit>".
	Duplicate Name, TempName
	Wave/T tw = TempName
	Variable numPoints = DimSize(tw, 0)
	Variable i = 0
	do
		// We assume that this is something like R980912A.R1 or R980912A.R23
		String newName = tw[i]
		Variable len = strlen(newName)

		// Is the next-to-last character a letter rather than a digit?
		if (char2num(newName[len-2]) > char2num("9"))
			newName = newName[0, len-2] + "0" + newName[len-1]
			tw[i] = newName
		endif

		i += 1
	while(i < numPoints)

	Sort {TempName,Start}, Name, Start, Stop, Average, StdDev, Slope, Pearson, YStart, YStop

	KillWaves/Z TempName

	SetDataFolder originalCurrentDataFolder
End

Function MakeMeasurementWaves(overwrite)
	Variable overwrite				// If 1, truncates existing waves to 0 points.

	String originalCurrentDataFolder
	originalCurrentDataFolder = GetDataFolder(1)

	if (!DataFolderExists("root:Packages:RatioImg:Measurements"))
		NewDataFolder root:Packages:RatioImg:Measurements
	endif
	SetDataFolder root:Packages:RatioImg:Measurements

	if ((overwrite!=0) + !WaveExists(Name))
		Make/O/T/N=0 Name					// e.g., "R981206B.R7"
	endif
	if ((overwrite!=0) + !WaveExists(Start))
		Make/O/N=0 Start						// Measurement start time.
	endif
	if ((overwrite!=0) + !WaveExists(Stop))
		Make/O/N=0 Stop						// Measurement stop time.
	endif
	if ((overwrite!=0) + !WaveExists(Average))
		Make/O/N=0 Average					// Average value.
	endif
	if ((overwrite!=0) + !WaveExists(StdDev))
		Make/O/N=0 StdDev					// Standard deviation.
	endif
	if ((overwrite!=0) + !WaveExists(Slope))
		Make/O/N=0 Slope						// Slope.
	endif
	if ((overwrite!=0) + !WaveExists(Pearson))
		Make/O/N=0 Pearson					// Pearson's r.
	endif
	if ((overwrite!=0) + !WaveExists(YStart))
		Make/O/N=0 yStart						// Y value at start time.
	endif
	if ((overwrite!=0) + !WaveExists(YStop))
		Make/O/N=0 yStop						// Y value at stop time.
	endif

	SetDataFolder originalCurrentDataFolder
End

Function DeleteAllMeasurements(ctrlName) : ButtonControl
	String ctrlName
	DoAlert 1, "This will clear all data from the Measurements table.  Are you sure you want to do this?"
	if (V_flag == 1)
		MakeMeasurementWaves(1)
		print "All measurements deleted."
	else
		print "Cancelled delete all measurements."
	endif
	
End

Function DeleteLastMeasurement(ctrlName) : ButtonControl
	String ctrlName
	String originalCurrentDataFolder
	originalCurrentDataFolder = GetDataFolder(1)

	if (!DataFolderExists("root:Packages:RatioImg:Measurements"))
		return -1
	endif
	SetDataFolder root:Packages:RatioImg:Measurements

	DeletePoints numpnts(Name)-1, 1, Name
	DeletePoints numpnts(Start)-1, 1, Start
	DeletePoints numpnts(Stop)-1, 1, Stop
	DeletePoints numpnts(Average)-1, 1, Average
	DeletePoints numpnts(StdDev)-1, 1, StdDev
	DeletePoints numpnts(Slope)-1, 1, Slope
	DeletePoints numpnts(Pearson)-1, 1, Pearson
	DeletePoints numpnts(YStart)-1, 1, YStart
	DeletePoints numpnts(YStop)-1, 1, YStop

	SetDataFolder originalCurrentDataFolder
	Print "Last set of measurements deleted."
End

//Function MeasureAllCellsForSegment (dataFolder, startCell, endCell) : ButtonControl
//	This routine measures the defined segment on a wave for each cell from startCell to endCell.
//	dataFolder is a RatioImg data folder containing A/B/R data loaded to be measured.

Function MeasureAllCellsForSegment (dataFolder, startCell, endCell) : ButtonControl
	String dataFolder									// The RatioImg data folder.
	Variable startCell, endCell							// The cells to be measured.
	return -1
	
	String graphName = WinName(0, 1)				// Name of the top graph.
	if (!IsRatioImgGraph(graphName))
		DoAlert 0, "Activate a RatioImg graph first."
		return -1
	endif

	String originalCurrentDataFolder
	originalCurrentDataFolder = GetDataFolder(1)
	SetDataFolder(dataFolder)
	
	Wave wA = CsrWaveRef(A)
	if (!WaveExists(wA))
		DoAlert 0, "Put cursor A on a wave first"
		return -1
	endif
	Wave wB = CsrWaveRef(B)
	if (!WaveExists(wB))
		DoAlert 0, "Put cursor B on a wave first"
		return -1
	endif
	
	DoAlert 1, "This will clear all data from the Measurements table.  Are you sure you want to do this?"
	if (V_flag == 1)
		MakeMeasurementWaves(1)
		
		if (startCell < 1)									// Clip startCell.
			startCell = 2									
		endif
	
		NVAR gNumDataPairs=gNumDataPairs				// Create a reference to the global variable in the current data folder.
		if (endCell > gNumDataPairs)						// Clip endCell.
			endCell = gNumDataPairs
		endif
		
//	// Get the data folder for the current run.
//	String dataFolder = GetDataFolderForDisplayedRun(1)		// Includes a trailing colon.
//	if (strlen(dataFolder) == 0)
//		CantFindRunAlert()
//		return -1
//	endif
	
		Variable i
	
		// Get a list of the waves to be measured.
		String oldWaveList = ""
		i = 0
		do
			WAVE w = WaveRefIndexed("", i, 1)						// Get a reference to the next wave in the top graph.
			if (!WaveExists(w))
				break												// No more waves.
			endif
			if (IsRatioImgWave(w))
				oldWaveList += GetWavesDataFolder(w, 2) + ";"	// This includes the name of the wave, not just the data folder name.
			endif
			i += 1
		while(1)
	
		if (strlen(oldWaveList) == 0)
			return -1												// No waves were found to replace.
		endif

		Variable cell = startCell
		do
	
	// Replace each wave with the next or previous wave of the same type (e.g., A2->A3, B2->B3, R2->R3).
		String traceName, oldWave
		i = 0
		oldWave = StringFromList(i, oldWaveList, ";")
		if (strlen(oldWave) == 0)							// No more waves to replace.
			break
		endif
		Wave oldW = $oldWave								// Get a reference to the old wave.
		traceName = WaveToTraceName("", oldW)			// Get the trace name for the wave.
		if (strlen(traceName) == 0)
			break											// Should never happen.
		endif
		Wave newW = $GetNextRatioImgWave(oldW, 1)
		if (WaveExists(newW))							// It should always exist.
			AddMeasurementFromCursors("")
			cell += 1
		endif
	while(cell <= endCell)
endif
		SetDataFolder(originalCurrentDataFolder)			// Restore current data folder.
End	

Function AddMeasurementFromCursors(ctrlName) : ButtonControl
	String ctrlName

	String graphName = WinName(0, 1)				// Name of the top graph.
	if (!IsRatioImgGraph(graphName))
		DoAlert 0, "Activate a RatioImg graph first."
		return -1
	endif

	String originalCurrentDataFolder
	originalCurrentDataFolder = GetDataFolder(1)

	Wave wA = CsrWaveRef(A)
	if (!WaveExists(wA))
		DoAlert 0, "Put cursor A on a wave first"
		return -1
	endif
	Wave wB = CsrWaveRef(B)
	if (!WaveExists(wB))
		DoAlert 0, "Put cursor B on a wave first"
		return -1
	endif
	
	SetDataFolder GetWavesDataFolder(wA,1)			// Do the fit in the wave's data folder so that waves created by the fit will not conflict with analogous waves from other runs.

	WaveStats/Q/R=(xcsr(A),xcsr(B)) wA
	Variable vavg = V_avg
	Variable vstddev = V_sdev

	CurveFit /Q line wA (xcsr(A),xcsr(B)) /D
	Wave wc = W_coef									//Create reference to coef wave made by CurveFit
	Wave ws = W_sigma								//Create reference to sigma wave made by CurveFit
	Variable vslope = wc[1]
	Variable vPearson = V_Pr
	KillWaves/Z wc, ws				// Remove clutter.
	
	String waveAName = NameOfWave(wA)				// e.g., R12
	String runName = GetWavesDataFolder(wA, 0)		// e.g., R981209A
	
	Variable n						// Number of pre-existing rows.
	
	MakeMeasurementWaves(0)						// Make data folder and waves if they do not yet exist.
	SetDataFolder root:Packages:RatioImg:Measurements

	Wave/T twOut = name
	n = DimSize(twOut, 0)
	InsertPoints n, 1, twOut
	twOut[n] = runName + "." + waveAName		// e.g., R981209A.R12

	Wave wOut = start
	n = DimSize(wOut, 0)
	InsertPoints n, 1, wOut
	wOut[n] = xcsr(A)

	Wave wOut = stop
	n = DimSize(wOut, 0)
	InsertPoints n, 1, wOut
	wOut[n] = xcsr(B)

	Wave wOut = average
	n = DimSize(wOut, 0)
	InsertPoints n, 1, wOut
	wOut[n] = vavg

	Wave wOut = stddev
	n = DimSize(wOut, 0)
	InsertPoints n, 1, wOut
	wOut[n] = vstddev

	Wave wOut = slope
	n = DimSize(wOut, 0)
	InsertPoints n, 1, wOut
	wOut[n] = vslope

	Wave wOut = pearson
	n = DimSize(wOut, 0)
	InsertPoints n, 1, wOut
	wOut[n] = vpearson

	Wave wOut = ystart
	n = DimSize(wOut, 0)
	InsertPoints n, 1, wOut
	wOut[n] = vcsr(A)

	Wave wOut = ystop
	n = DimSize(wOut, 0)
	InsertPoints n, 1, wOut
	wOut[n] = vcsr(B)

	Print "Name =", runName + "." + waveAName, "| Start =", xcsr(A), "| Stop =", xcsr(B), "| Average =", V_avg, "| Std Dev =", vstddev, "| Slope =", vslope, "| r =", vPearson, "| A(Y) =", vcsr(A), "| B(Y) =", vcsr(B)

	SetDataFolder originalCurrentDataFolder
End

Function RemoveFittedLines()		// Removes and kills waves named fit_<whatever>
	String graphName = WinName(0, 1)				// Name of the top graph.
	if (!IsRatioImgGraph(graphName))
		DoAlert 0, "Activate a RatioImg graph first."
		return -1
	endif

	Variable i
	String name, traceName
	i = 0
	do
		Wave w = WaveRefIndexed("", i, 1)
		if (!WaveExists(w))
			break							// All done.
		endif
		name = NameOfWave(w)
		if (CmpStr(name[0,3], "fit_") == 0)
			traceName = WaveToTraceName("", w)
			if (strlen(traceName) > 0)					// Should always be > 0.
				RemoveFromGraph $traceName
				KillWaves/Z w
				i -= 1
			endif
		endif
		i += 1
	while(1)
End
