#pragma rtGlobals=1										// Use modern global access method.

Function/S RemoveTrailingColon(str)						// Remove trailing colon if it is there.
	String str
	
	Variable len = strlen(str)
	if (len == 0)
		return str
	endif
	
	if (CmpStr(str[len-1], ":") == 0)
		str = str[0, len-2]
	endif
	
	return str
End

Function/S AddTrailingColon(str)							// Adds a trailing colon if it is not already there.
	String str

	Variable len = strlen(str)
	if (CmpStr(str[len-1], ":") != 0)
		str += ":"
	endif

	return str
End

//	GetLeafName(pathStr)
//		Returns the "leaf" of a path.
//		For example, if the path is "hd:Folder A:Folder B:File A", this returns "File A".
//		The path separator is assumed to be colon.
//		If no leaf name is found, returns "".
//		The input path may end with a colon or it may be omitted.
Function/S GetLeafName(pathStr)
	String pathStr
	
	pathStr = RemoveTrailingColon(pathStr)				// Remove trailing colon if it is there.
	
	String leafName
	Variable pos
	
	leafName = ""
	pos = strlen(pathStr) - 1								// Search from the end
	do
		if (CmpStr(pathStr[pos], ":") == 0)				// Found the colon?
			leafName = pathStr[pos+1, strlen(pathStr)]
			break											// Break out of loop.
		endif
		pos -= 1
	while(pos > 0)
	
	return leafName
End

//	GetParentDataFolder(dataFolder, getFullPath)
//	dataFolder is a path to a data folder (e.g., "root:FolderA:FolderB"). It may end with
//	a trailing colon or not.
//	Returns the parent of the specified data folder.
//	If getFullPath is 0, it returns just the parent data folder name.
//	If getFullPath is 1, it returns the full path to the parent, including a trailing colon. In this
//	case, dataFolder must be a full path on input.
Function/S GetParentDataFolder(dataFolder, getFullPath)
	String dataFolder
	Variable getFullPath

	Variable len
	
	dataFolder = RemoveTrailingColon(dataFolder)		// Remove trailing colon if it is there.
	len = strlen(dataFolder)
	
	String leafName
	leafName = GetLeafName(dataFolder)
	if (strlen(leafName) == 0)
		return ""										// Not a valid data folder.
	endif
	
	// Chop off the leaf name.
	dataFolder = dataFolder[0, len-strlen(leafName)-1]

	len = strlen(dataFolder)
	if (CmpStr(dataFolder[len-1], ":") != 0)
		return ""										// There is no parent data folder.
	endif
	
	if (getFullPath)
		return dataFolder								// Full path including trailing colon.
	endif
	
	// Get just the parent name.	
	dataFolder = dataFolder[0, len-2]					// Get rid of trailing colon if any.
	len -= 1
	leafName = GetLeafName(dataFolder)
	return leafName	
End

Function/S UniqueListItems(list1, list2)				// Returns a list of items in list2 that are not in list1.
	String list1, list2
	
	String outputList
	String item
	Variable i

	outputList = ""
	i = 0
	do
		item = StringFromList(i, list2, ";")				// Get item from list2.
		if (strlen(item) == 0)
			break
		endif
		if (strsearch(list1, item + ";", 0) < 0)			// Item not in list1?
			outputList += item + ";"
		endif
		i += 1
	while(1)
	
	return outputList
End

//	GetBrowserSelectionNoQuotes(index)
//	Returns the indexed data browser selection without single quotes.
//	Use this instead of the GetBrowserSelection function that is provided by the Data Browser.
//	GetBrowserSelection always quotes the leaf name even if it does not need to
//	be quoted. This causes a problem when using UniqueListItems to avoid including the
//	same data folder twice. The problem occurs because the same data folder might appear
//	twice but with different quoting. By removing the quotes added by GetBrowserSelection,
//	we avoid this problem, but also make it so that these procedures won't work with liberal
//	names. This is OK because liberal names are too difficult to program with to be useful anyway.
Function/S GetBrowserSelectionNoQuotes(index)
	Variable index
	
	String item
	item = GetBrowserSelection(index)
	
	String output = ""
	String val
	Variable pos, len
	len = strlen(item)
	pos = 0
	do
		if (pos >= len)
			break
		endif
		val = item[pos]							// Get next character.
		if (CmpStr(val, "'") != 0)				// Is it a single quote?
			output += val						// Add to output if not.
		endif
		pos += 1	
	while(1)
	
	return output	
End

//	GetSubDataFolders(dataFolder, recurse)
//	Returns a semi-colon-separated list of all data folders inside the specified data folder.
//	If recurse is non-zero, it recursively includes all data folders at lower levels.
Function/S GetSubDataFolders(dataFolder, recurse)
	String dataFolder							// The parent data folder.
	Variable recurse							// True if we want to get grandchildren and great-grandchildren.
	
	String outputList, nextFolder
	Variable i
	
	dataFolder = AddTrailingColon(dataFolder)	// GetIndexedObjName requires a path that ends with a colon.
	
	outputList = ""
	i = 0
	do
		nextFolder = GetIndexedObjName(dataFolder, 4, i)	// Get the next data folder within the input data folder.
		if (strlen(nextFolder) == 0)
			break											// No more data folders.
		endif
		outputList += dataFolder + nextFolder + ";"
		if (recurse)
			// Recursively add nested data folders.
			outputList += GetSubDataFolders(dataFolder + nextFolder, 1)
		endif
		i += 1
	while(1)
	
	return outputList
End

//	GetSelectedDataFolders(includeSubDataFolders)
//	Returns a list of the data folders selected in the Data Browser.
//	If includeSubDataFolders is true, it includes all data folders nested within the
//	selected data folders.
Function/S GetSelectedDataFolders(includeSubDataFolders)
	Variable includeSubDataFolders

	String outputList = ""
	Variable i
	String nextFolder

	i = 0
	do
		nextFolder = GetBrowserSelectionNoQuotes(i)
		if (strlen(nextFolder) == 0)
			break
		endif
		if (DataFolderExists(nextFolder))									// Make sure it's a data folder and not a wave or variable.
			outputList += UniqueListItems(outputList, nextFolder + ";")		// Add data folder if not already in the list.
			if (includeSubDataFolders)
				String list = GetSubDataFolders(nextFolder, 1) 				// Get sub data folder list.
				outputList += UniqueListItems(outputList, list)				// Add items that are not already in the list.
			endif
		endif
		i += 1
	while(1)

	return outputList
End

//	IsRunDataFolder(dataFolder)
//	Returns the truth that the data folder is a run data folder.
//	We assume that a data folder is a run data folder if the name is of the form "R<date>"
//	where <date> is any six-digit number.
Function IsRunDataFolder(dataFolder)
	String dataFolder
	
	String leafName, numericPart, temp
	Variable num
	
	if (DataFolderExists(dataFolder) == 0)
		return 0									// Input data folder is bogus.
	endif
	
	leafName = GetLeafName(dataFolder)
	if (strlen(leafName) < 7)
		return 0
	endif
	
	if (CmpStr(leafName[0,0], "R") != 0)
		return 0									// Name does not start with "R".
	endif
	
	// Look for the number after "R".
	numericPart = leafName[1, 6]					// This must be six digits.
	Variable i = 0
	do
		Variable digit = char2num(numericPart[i]) - char2num("0")	// Convert digit character to number from 0 to 9.
		if ( (digit < 0) %| (digit > 9) )
			return 0								// Character is not a digit.
		endif
		i += 1
	while(i < 6)	
	
	return 1
End

//	GetSelectedRunDataFolders(includeSubDataFolders)
//	Returns a list of the run data folders selected in the Data Browser. See IsRunDataFolder for
//	an explanation of the naming convention for run data folders.
//	If includeSubDataFolders is true, it includes all run data folders nested within the
//	selected data folders. In this case, if a run data folder is selected, this will return
//	all enclosed run data folders.
//	Functions that operate on the selected run data folders call GetSelectedRunDataFolders(1)
//	to get the list of data folders.	
Function/S GetSelectedRunDataFolders(includeSubDataFolders)
	Variable includeSubDataFolders
	
	String list
	list = GetSelectedDataFolders(includeSubDataFolders)

	// Now cull out those that are not run data folders.
	String nextFolder, leafName
	Variable i
	String outputList	
	
	i = 0
	outputList = ""
	do
		nextFolder = StringFromList(i, list, ";")
		if (strlen(nextFolder) == 0)
			break
		endif
		if (IsRunDataFolder(nextFolder))
			outputList += nextFolder + ";"
		endif
		i += 1
	while(1)
	
	return outputList
End

//	GetRunDataFolders(parentDataFolder, recurse)
//	Returns a list of the run data folders selected inside the specified parent data folder.
//	See IsRunDataFolder for an explanation of the naming convention for run data folders.
//	If recurse is true, it includes all run data folders nested within the
//	parent data folder. If recurse is false, it returns only those run data folders at the
//	top level of the parent data folder.
Function/S 	GetRunDataFolders(parentDataFolder, recurse)
	String parentDataFolder
	Variable recurse
	
	String dataFolderList = GetSubDataFolders(parentDataFolder, recurse)
	
	String output = ""
	Variable i = 0
	do
		String dataFolder = StringFromList(i, dataFolderList, ";")
		if (strlen(dataFolder) == 0)
			break
		endif
		if (IsRunDataFolder(dataFolder))
			output += dataFolder + ";"
		endif
		i += 1
	while(1)
	
	return output
End

//	GetNextRunDataFolder(runDataFolder)
//	runDataFolder is the full path to a data folder containing a run. It may or may not contain a trailing colon.
//	GetNextRunDataFolder returns the next run data folder at the same level of the hierarchy.
//	If wrap is true, it wraps around if runDataFolder is the last run data folder. If wrap is false,
//	it returns "" if runDataFolder is the last data folder.
Function/S GetNextRunDataFolder(runDataFolder, wrap)
	String runDataFolder				// Data folder containing a run.
	Variable wrap
	
	runDataFolder = RemoveTrailingColon(runDataFolder)
	
	String parentDataFolder = GetParentDataFolder(runDataFolder, 1)
	if (strlen(parentDataFolder) == 0)
		return ""						// Should not happen.
	endif

	String dataFolderList = GetRunDataFolders(parentDataFolder, 0)		// List of all run data folders at this level.
	if (strlen(dataFolderList) == 0)
		return ""														// Should not happen.
	endif
	
	String dataFolder
	Variable foundInputRunDataFolder = 0			// Set to true when we hit the input data folder.
	Variable i = 0
	do
		dataFolder = StringFromList(i, dataFolderList, ";")
		if (strlen(dataFolder) == 0)				// Hit the end?
			if (foundInputRunDataFolder)			// Need to wrap?
				if (wrap)
					dataFolder = StringFromList(0, dataFolderList, ";")
				endif
			endif
			break
		endif
		if (IsRunDataFolder(dataFolder))			// Skip any other garbage in the parent.
			if (foundInputRunDataFolder)
				break								// dataFolder is the next one.
			endif
			if (CmpStr(runDataFolder, dataFolder) == 0)
				foundInputRunDataFolder = 1		// We will take the next one.
			endif
		endif
		
		i += 1
	while(1)
	
	return dataFolder
End

//	GetPrevRunDataFolder(runDataFolder)
//	runDataFolder is the full path to a data folder containing a run. It may or may not contain a trailing colon.
//	GetPrevRunDataFolder returns the previous run data folder at the same level of the hierarchy.
//	If wrap is true, it wraps around if runDataFolder is the first run data folder. If wrap is false,
//	it returns "" if runDataFolder is the first data folder.
Function/S GetPrevRunDataFolder(runDataFolder, wrap)
	String runDataFolder				// Data folder containing a run.
	Variable wrap
	
	runDataFolder = RemoveTrailingColon(runDataFolder)
	
	String parentDataFolder = GetParentDataFolder(runDataFolder, 1)
	if (strlen(parentDataFolder) == 0)
		return ""						// Should not happen.
	endif

	String dataFolderList = GetRunDataFolders(parentDataFolder, 0)		// List of all run data folders at this level.
	if (strlen(dataFolderList) == 0)
		return ""														// Should not happen.
	endif
	
	// Determine the total number of run data folders in the list.
	Variable n = 0
	do
		String temp = StringFromList(n, dataFolderList, ";")
		if (strlen(temp) == 0)
			break
		endif
		n += 1
	while(1)
	
	String dataFolder
	Variable foundInputRunDataFolder = 0			// Set to true when we hit the input data folder.
	Variable i = n-1								// Loop from the last index down to 0.
	do
		dataFolder = StringFromList(i, dataFolderList, ";")
		if (IsRunDataFolder(dataFolder))			// Skip any other garbage in the parent.
			if (foundInputRunDataFolder)
				break								// dataFolder is the next one.
			endif
			if (CmpStr(runDataFolder, dataFolder) == 0)
				foundInputRunDataFolder = 1		// We will take the next one.
			endif
		endif
		
		i -= 1

		if (i == -1)								// Hit the beginning?
			if (foundInputRunDataFolder)			// Need to wrap?
				if (wrap)
					dataFolder = StringFromList(n-1, dataFolderList, ";")
				else
					dataFolder = ""
				endif
			endif
			break
		endif
	while(1)
	
	return dataFolder
End

//	IsRatioImgGraph(graphName)
//	Returns the truth that the graph is a RatioImg graph.
Function IsRatioImgGraph(graphName)
	String graphName
	
	if (CmpStr(graphName[0, 12], "RatioImgGraph") == 0)
		return 1
	endif
	return 0
End

//	WaveToTraceName(graphNameStr, theWave)
//	graphNameStr is "" for the top graph or the name of a graph.
//	theWave is a reference to a particular wave that is displayed in the graph.
//	WaveToTraceName returns the trace name for that wave. The trace name is
//	needed for the ModifyGraph and RemoveFromGraph operations.
//	This implementation is designed to handle the case were multiple waves with the
//	same name but in different data folders are displayed in the graph. It does not
//	handle the case where the same exact wave is displayed multiple times in a graph.
Function/S WaveToTraceName(graphNameStr, theWave)
	String graphNameStr				// Name of graph or "" for top graph.
	Wave theWave						// Reference to a wave in the graph for which we want the trace name.
	
	if (WaveExists(theWave) == 0)
		return ""						// Bad input.
	endif

	String fullPathToWave1, fullPathToWave2
	fullPathToWave1 = GetWavesDataFolder(theWave, 1) + NameOfWave(theWave)
	
	String traceList = TraceNameList(graphNameStr, ";", 1)
	String traceName
	
	// Loop through the traces calling TraceNameToWaveRef until we find which trace corresponds to theWave.
	Variable i = 0
	do
		traceName = StringFromList(i, traceList, ";")
		if (strlen(traceName) == 0)
			break													// No more traces.
		endif
		
		WAVE w = TraceNameToWaveRef("", traceName)
		if (WaveExists(w) == 0)
			traceName = ""
			break													// Should not happen.
		endif
		fullPathToWave2 = GetWavesDataFolder(w,1) + NameOfWave(w)
				
		if (CmpStr(fullPathToWave1, fullPathToWave2) == 0)		// We found the trace corresponding to the specified wave?
			break
		endif
		
		i += 1
	while(1)
	
	return traceName
End	

//	GetDataFolderForDisplayedRun(doAlertIfNoActiveRunFound)
//	Returns the full path to the data folder for the run displayed in the active graph.
//	The returned data folder path includes a trailing semicolon.
//	If there are no graphs or no runs in the active graph, returns "".
Function/S GetDataFolderForDisplayedRun(doAlertIfNoActiveRunFound)
	Variable doAlertIfNoActiveRunFound			// If true, we display an alert if we can't find a RatioImg graph or a RatioImg wave in the graph.

	String graphName, dataFolder
	
	graphName = WinName(0, 1)				// Get the name of the active graph.
	if (strlen(graphName) == 0)
		if (doAlertIfNoActiveRunFound)
			DoAlert 0, "Can't find run data in the active graph"
		endif
		return ""								// There are no graphs.
	endif
	
	Variable i = 0
	do
		Wave w = WaveRefIndexed("", i, 1)		// Get reference to the next wave in the graph.
		if (!WaveExists(w))
			if (doAlertIfNoActiveRunFound)
				DoAlert 0, "Can't find run data in the active graph"
			endif
			return ""							// Did not find a run wave.
		endif
		dataFolder = GetWavesDataFolder(w, 1)	// This includes a trailing colon.
		if (IsRunDataFolder(dataFolder))
			return dataFolder					// Success.
		endif
		
		i += 1
	while(1)					// Continue till we hit one of the returns above.
End

//	IsRatioImgWave(w)
//	Returns 1 if the wave is a RatioImg wave or 0 otherwise.
//	A RatioImg wave is defined as a wave that is stored in a run data folder and whose name
//	is of the form Ai, Bi, or Ri.
Function IsRatioImgWave(w)
	Wave w
	
	if (!WaveExists(w))
		return 0
	endif

	String dataFolder
	dataFolder = GetWavesDataFolder(w, 1)
	if (!IsRunDataFolder(dataFolder))
		return 0
	endif
	
	String name = NameOfWave(w)
	Variable len = strlen(name)
	if (len < 2)
		return 0							// Name must be at least two characters long.
	endif
	
	String char = name[0]
	if ( (CmpStr(char,"A")!=0) %& (CmpStr(char,"B")!=0) %& (CmpStr(char,"R")!=0) )
		return 0							// Does not start with A, B or R.
	endif
	
	// The remaining characters in the name must be digits.
	Variable digit
	Variable i = 1
	do
		char = name[i]
		digit = char2num(char) - char2num("0")
		if ( (digit < 0) %| (digit > 9) )
			return 0
		endif
		i += 1		
	while(i < len)
	
	return 1
End

//	IsRatioImgWaveOfType(w, typeStr)
//	Returns the true that the specified wave is a RatioImg wave of the specified type.
//	typeStr is one of the following: "A", "B", "R".
Function IsRatioImgWaveOfType(w, typeStr)
	Wave w
	String typeStr
	
	if (!IsRatioImgWave(w))
		return 0
	endif
	
	String name = NameOfWave(w)
	if (CmpStr(name[0], typeStr) == 0)
		return 1
	endif
	
	return 0	
End

//	GetRatioImgWaveCellNumber(w)
//	Returns the cell number for the specified wave which must be an Ai, Bi, or Ci wave.
//	Returns the cell number or 0 if this is not a valid wave.
Function GetRatioImgWaveCellNumber(w)
	Wave w
	
	if (!IsRatioImgWave(w))
		return 0
	endif
	
	String name = NameOfWave(w)
	Variable cell = str2num(name[1,INF])	
	return cell
End

//	GetNextRatioImgWave(w, wrap)
//	Returns the full path to the next wave of the same type. For example A2->A3, B2->B3, R2->R3.
//	If w is the last wave in the run and wrap is true, it returns the first wave in the run.
//	If w is the last wave in the run and wrap is false, it returns "".
Function/S GetNextRatioImgWave(w, wrap)
	Wave w
	Variable wrap
	
	if (!IsRatioImgWave(w))
		return ""					// Input wave is bogus.
	endif
	
	String dataFolder = GetWavesDataFolder(w,1)				// Includes trailing colon.
	
	String name = NameOfWave(w)
	Variable cell = GetRatioImgWaveCellNumber(w)					// Pick of the number after the A, B, R.
	
	String nextName = name[0] + num2istr(cell+1)			// A2->A3, B2->B3, R2->R3.
	Wave nextW = $(dataFolder + nextName)					// Get reference to the next wave.
	if (!WaveExists(nextW))
		// We ran off the end of the waves.
		if (!wrap)
			return ""
		endif
		nextName = name[0] + "2"								// Wrap around to the first wave.
		Wave nextW = $(dataFolder + nextName)
		if (!WaveExists(nextW))
			return ""											// Should not happen.
		endif
	endif
	
	return dataFolder + nextName
End

//	GetPrevRatioImgWave(w, wrap)
//	Returns the full path to the previous wave of the same type. For example A3->A2, B3->B2, R3->R2.
//	If w is the first wave in the run and wrap is true, it returns the last wave in the run.
//	If w is the first wave in the run and wrap is fals, it returns "".
Function/S GetPrevRatioImgWave(w, wrap)
	Wave w
	Variable wrap
	
	if (!IsRatioImgWave(w))
		return ""					// Input wave is bogus.
	endif
	
	String dataFolder = GetWavesDataFolder(w,1)				// Includes trailing colon.
	NVAR numDataPairs = $(dataFolder + "gNumDataPairs")		// Create a reference to the global variable containing the number of A/B pairs in the run.
	
	String name = NameOfWave(w)
	Variable cell = GetRatioImgWaveCellNumber(w)					// Pick of the number after the A, B, R.
	
	String prevName = name[0] + num2istr(cell-1)			// A3->A2, B3->B2, R3->R2.
	Wave prevW = $(dataFolder + prevName)					// Get reference to the previous wave.
	if (!WaveExists(prevW))
		// We ran off the end of the waves.
		if (!wrap)
			return ""
		endif
		prevName = name[0] + num2istr(numDataPairs)		// Wrap around to the last wave.
		Wave prevW = $(dataFolder + prevName)
		if (!WaveExists(prevW))
			return ""											// Should not happen.
		endif
	endif
	
	return dataFolder + prevName
End
